searchNodes=[{"doc":"InfluxDB driver for Elixir Connections To connect to an InfluxDB server you need a connection module: defmodule MyConnection do use Instream.Connection , otp_app : :my_app end The :otp_app name and the name of the module can be freely chosen but have to be linked to a corresponding configuration entry. This defined connection module needs to be hooked up into your supervision tree: children = [ # ... MyConnection , # ... ] Example of the matching configuration entry: # InfluxDB v2.x config :my_app , MyConnection , auth : [ method : :token , token : &quot;my_token&quot; ] , bucket : &quot;my_default_bucket&quot; , org : &quot;my_default_org&quot; , host : &quot;my.influxdb.host&quot; , version : :v2 # InfluxDB v1.x config :my_app , MyConnection , auth : [ username : &quot;my_username&quot; , password : &quot;my_password&quot; ] , database : &quot;my_default_database&quot; , host : &quot;my.influxdb.host&quot; More details on connections and configuration options can be found with the modules Instream.Connection and Instream.Connection.Config . Queries To read data from your InfluxDB server you should send a query: # Flux query MyConnection . query ( ~s( from(bucket: &quot; \#{ MyConnection . config ( :bucket ) } &quot;) |&gt; range ( start : - 5 m ) |&gt; filter ( fn : ( r ) =&gt; r . _measurement == &quot;instream_examples&quot; ) ) ) # InfluxQL query MyConnection . query ( &quot;SELECT * FROM instream_examples&quot; ) Most of the queries you send require a :database or :bucket / :organization to operate on. These values will be taken from your connection configuration by default. By using the option argument of MyConnection.query/2 you can pass different values to use on a per-query basis: MyConnection . query ( &quot;... query ...&quot; , bucket : &quot;my_other_bucket&quot; , org : &quot;my_other_organization&quot; ) MyConnection . query ( &quot;... query ...&quot; , database : &quot;my_other_database&quot; ) Responses from a query will be decoded into maps by default. Depending on your InfluxDB version you can use the :result_as option argument to skip the decoding or request a non-default response type: result_as: :csv : response as CSV when using InfluxDB v1 result_as: :raw : result as sent from the server without decoding Query Language Selection Depending on your configured InfluxDB version all queries will be treated as :flux (v2) or :influxql by default. You can send a query in the non-default language by passing the :query_language option: MyConnection . query ( &quot;... query ...&quot; , query_language : :flux ) MyConnection . query ( &quot;... query ...&quot; , query_language : :influxql ) Query Parameter Binding (InfluxDB v1.x) Queries can be parameterized, for example when you are dealing with untrusted user input: MyConnection . query ( &quot;SELECT * FROM some_measurement WHERE field = $field_param&quot; , params : %{ field_param : &quot;some_value&quot; } ) POST Queries (InfluxDB v1.x) Some queries require you to switch from the regular read only context (all GET requests) to a write context (all POST requests). When not using the query build you have to pass that information manually to query/2 : MyConnection . query ( &quot;CREATE DATABASE create_in_write_mode&quot; , method : :post ) Query Timeout Configuration If you find your queries running into timeouts (e.g. :hackney not waiting long enough for a response) you can pass an option to the query call: MyConnection . query ( query , http_opts : [ recv_timeout : 250 ] ) This value can also be set as a default using your HTTP client configuration . A passed configuration will take precedence over the connection configuration. Writing Points Writing data to your InfluxDB server is done using either Instream.Series modules or raw maps. Depending on your connection configuration the selected writer module provides additional options. The write function can be used with a single or multiple data points: MyConnection . write ( point ) MyConnection . write ( [ point_1 , point_2 ] ) Writing Points using Series Each series in your database can be represented using a definition module: defmodule MySeries do use Instream.Series series do measurement &quot;my_measurement&quot; tag :bar tag :foo field :value end end This module will provide you with a struct you can use to define points you want to write to your database: MyConnection . write ( % MySeries { fields : % MySeries.Fields { value : 17 } , tags : % MySeries.Tags { bar : &quot;bar&quot; , foo : &quot;foo&quot; } } ) More information about series definitions can be found in the module documentation of Instream.Series . Writing Points using Plain Maps As an alternative you can use a non-struct map to write points to a database: MyConnection . write ( %{ measurement : &quot;my_measurement&quot; , fields : %{ answer : 42 , value : 1 } , tags : %{ foo : &quot;bar&quot; } , timestamp : 1_439_587_926_000_000_000 } , # more points possible ... ) The field :timestamp is optional. InfluxDB will use the receive time of the write request if it is missing.","ref":"Instream.html","title":"Instream","type":"module"},{"doc":"Defines a connection to an InfluxDB instance. Connection Definition defmodule MyConnection do use Instream.Connection , otp_app : :my_app end This connection will fetch its configuration from the application environment as defined by :otp_app . As an alternative you can define the configuration in the module definition itself: defmodule MyConnection do use Instream.Connection , config : [ version : :v1 , host : &quot;my.influxdb.host&quot; , scheme : &quot;http&quot; ] end Both inline and :otp_app configuration can be mixed. In this case the application configuration will overwrite any inline values. For more information on how to configure your connection please refer to the documentation of Instream.Connection.Config . Connection Configuration There are some configuration values that should be checked/changed to get your connection up and running: :auth : the authentication method and credentials :host : the hostname of the server (defaults to &quot;localhost&quot; ) :port : the port of the server (defaults to 8086 ) :version : the InfluxDB server version you are using ( :v1 or :v2 ) Some additional configuration options/requirements depend on the used version: :org : InfluxDB v2.x organization :bucket : InfluxDB v2.x bucket :database : InfluxDB v1.x database InfluxDB v2.x Compatibility Endpoint (InfluxQL Queries) If you are using InfluxQL queries with a :v2 connection you need to set the :database configuration to a pre-mapped database. Please refer to the official InfluxDB DBRP mapping documentation for details.","ref":"Instream.Connection.html","title":"Instream.Connection","type":"behaviour"},{"doc":"Returns a supervisable connection child_spec.","ref":"Instream.Connection.html#c:child_spec/1","title":"Instream.Connection.child_spec/1","type":"callback"},{"doc":"Returns the connection configuration.","ref":"Instream.Connection.html#c:config/1","title":"Instream.Connection.config/1","type":"callback"},{"doc":"Deletes data from an InfluxDB bucket. Usable options depend on the delete module configured.","ref":"Instream.Connection.html#c:delete/2","title":"Instream.Connection.delete/2","type":"callback"},{"doc":"Pings the connection server.","ref":"Instream.Connection.html#c:ping/1","title":"Instream.Connection.ping/1","type":"callback"},{"doc":"Executes a reading query. Options: database : use a database differing from the connection config for reading method : whether to use a :get or :post request org : use an organization differing from the connection config for reading precision : return data with a &quot;precision&quot; other than :rfc3339","ref":"Instream.Connection.html#c:query/2","title":"Instream.Connection.query/2","type":"callback"},{"doc":"Checks the status of the connection server. Only available with InfluxDB v1.x connections.","ref":"Instream.Connection.html#c:status/1","title":"Instream.Connection.status/1","type":"callback"},{"doc":"Determines the version of the connection server. If the version if undetectable (no header returned) it will be reported as &quot;unknown&quot; . If the host is unreachable or an error occurred the response will be :error .","ref":"Instream.Connection.html#c:version/1","title":"Instream.Connection.version/1","type":"callback"},{"doc":"Executes a writing query. Usable options depend on the writer module configured.","ref":"Instream.Connection.html#c:write/2","title":"Instream.Connection.write/2","type":"callback"},{"doc":"","ref":"Instream.Connection.html#t:e_version_mismatch/0","title":"Instream.Connection.e_version_mismatch/0","type":"type"},{"doc":"","ref":"Instream.Connection.html#t:log_entry/0","title":"Instream.Connection.log_entry/0","type":"type"},{"doc":"","ref":"Instream.Connection.html#t:precision/0","title":"Instream.Connection.precision/0","type":"type"},{"doc":"Configuration helper module. How To Configure There are multiple ways to configure a connection: Application Configuration: config files Runtime/Startup Configuration: functions modifying the application configuration Inline Configuration: values known at compile time All three can be mixed as required. Application Configuration If you know all configuration values before starting your application you can use config files (e.g. config.exs , release.exs and/or runtime.exs ) to set up your connection(s): config :my_app , MyConnection , database : &quot;my_default_database&quot; , host : &quot;my.influxdb.host&quot; , port : 8086 Runtime/Startup Configuration An alternative to config files is using an initializer function that will be called every time your connection is started (or restarted) in your supervision tree: config :my_app , MyConnection , init : { MyInitModule , :my_init_fun } config :my_app , MyOtherConnection , init : { MyInitModule , :my_init_fun , [ :extra , :args ] } defmodule MyInitModule do @spec my_init_fun ( module ) :: :ok def my_init_fun ( conn ) , do : my_init_fun ( conn , :extra , :args ) @spec my_init_fun ( module , atom , atom ) :: :ok def my_init_fun ( conn , :extra , :args ) do config = Keyword . merge ( conn . config ( ) , host : &quot;my.influxdb.host&quot; , port : 64210 ) Application . put_env ( :my_app , conn , config ) end end When the connection is started the function will be called with the connection module as the first argument. The function is expected to always return :ok . Inline Configuration In some environments it is sufficient to define the configuration in the connection module itself, for example during tests: defmodule MyConnection do use Instream.Connection , config : [ host : &quot;my.influxdb.host&quot; , port : 8086 ] end These values will be overwritten by and/or merged with the application environment values (if available) when the configuration is accessed. Configuration Defaults The following values will be used as defaults if no other value is set: config :my_app , MyConnection , version : :v1 , host : &quot;localhost&quot; , port : 8086 , scheme : &quot;http&quot; , http_client : Instream.HTTPClient.Hackney , loggers : [ { Instream.Log.DefaultLogger , :log , [ ] } ] , writer : Instream.Writer.Line , deleter : Instream.Deleter.Predicate , json_decoder : { Jason , :decode! , [ [ keys : :atoms ] ] } , json_encoder : { Jason , :encode! , [ ] } This also means that per default the connection uses no authentication. InfluxDB Version By default ( version: :v1 ) the communication will be done with the expectation of an InfluxDB 1.x server responding. If you are communicating with an InfluxDB 2.x server please configure version: :v2 . Any methods not supported by your configured version will respond with a special error tuple {:error, :version_mismatch} when called. HTTP Client Internally all requests are done using the configured :http_client . Instream.HTTPClient.Hackney Default for not otherwise configured HTTP Client is :hackney . The configuration key :http_opts is directly passed to the client process. Parts of it are also used internally by :hackney to control more generic behaviour (request pool to be used and its configuration). Please see :hackney.request/5 for a complete list of available options. Setting the :http_opts key when calling a connection method allows usage of per-call options. The options are merged with the connection options and then passed on. Unix Socket Connections Some InfluxDB versions allow connecting via a unix socket. These sockets have native support in :hackney with the appropriate configuration: config :my_app , MyConnection , scheme : &quot;http+unix&quot; , host : URI . encode_www_form ( &quot;/path/to/influxdb.sock&quot; ) Please be aware that you need to encode the socket path yourself. JSON Library By default the library used for encoding/decoding JSON is Jason . For the time :instream directly depends on it to ensure it is available. If you want to use another library you can switch it: config :my_app , MyConnection , json_decoder : MyJSONLibrary , json_encoder : MyJSONLibrary config :my_app , MyConnection , json_decoder : { MyJSONLibrary , :decode_argless } , json_encoder : { MyJSONLibrary , :decode_argless } config :my_app , MyConnection , json_decoder : { MyJSONLibrary , :decode_it , [ [ keys : :atoms ] ] } , json_encoder : { MyJSONLibrary , :decode_it , [ ] } If you configure only a module name it will be called as module.decode!(binary) and module.encode!(map) . When using a more complete {m, f} or {m, f, args} configuration the data to decode/encode will passed as the first argument with your configured extra arguments following. Authentication To connect to an InfluxDB instance with HTTP authentication enabled you have to configure your credentials: config :my_app , MyConnection , auth : [ method : :basic , username : &quot;root&quot; , password : &quot;root&quot; ] config :my_app , MyConnection , auth : [ method : :token , token : &quot;Ln0quM0YVQcJilrp&quot; ] For method you can choose between header authentication using :basic (InfluxDB v1) or :token (InfluxDB v2), or query parameters using :query (InfluxDB v1). If nothing or an invalid value is given the connection will be made using :basic authentication. Point Writer If you are using the regular line protocol writer Instream.Writer.Line you are done without having anything to configure. It is used by default and connects to the port you have configured for connection. To write points over UDP you can adjust your configuration: config :my_app , MyConnection , port_udp : 8089 , writer : Instream.Writer.UDP The connection will then write using UDP and connecting to the port :port_udp . All non-write queries will be send to the regular :port you have configured. Logging All queries are (by default) logged using Logger.debug/1 via the default logging module Instream.Log.DefaultLogger . To customize logging you have to alter the configuration of your connection: config :my_app , MyConnection , loggers : [ { FirstLogger , :log_fun , [ ] } , { SecondLogger , :log_fun , [ :additional , :args ] } ] This configuration replaces the default logging module. Configuration is given as a tuple of {module, function, arguments} . The log entry will be inserted as the first argument of the method call. It will be one of Instream.Log.PingEntry , Instream.Log.QueryEntry , Instream.Log.StatusEntry or Instream.Log.WriteEntry , depending on what type of request should be logged. In addition to query specific information every entry carries metadata around: :query_time : milliseconds it took to send request and receive the response :response_status : status code or 0 if not applicable/available When using the default logger you should re-configure :logger to be able to get them printed: config :logger , :console , format : &quot; $time $metadata[$level] $message &quot;, metadata : [ :application , :pid , :query_time , :response_status ] To prevent a query from being logged you can pass an option: MyConnection . ping ( log : false ) MyConnection . query ( query , log : false )","ref":"Instream.Connection.Config.html","title":"Instream.Connection.Config","type":"module"},{"doc":"Returns the configuration for a connection. The configuration will be merged with global defaults and the defaults passed to the lookup function. The latter are usually the inline defaults from the connection module ( conn ). If an otp_app is passed it will be used to fetch configuration values from the application environment. Priority for configuration lookup (first found is returned): Application configuration Inline configuration Global defaults","ref":"Instream.Connection.Config.html#get/4","title":"Instream.Connection.Config.get/4","type":"function"},{"doc":"Point deleter behaviour.","ref":"Instream.Deleter.html","title":"Instream.Deleter","type":"behaviour"},{"doc":"Deletes a point.","ref":"Instream.Deleter.html#c:delete/3","title":"Instream.Deleter.delete/3","type":"callback"},{"doc":"Optional list of workers to be supervised by the connection.","ref":"Instream.Deleter.html#c:deleter_workers/1","title":"Instream.Deleter.deleter_workers/1","type":"callback"},{"doc":"Point deleter for the line protocol. InfluxDB v2.x Options bucket : delete data to a specific bucket org : delete data to a specific organization","ref":"Instream.Deleter.Predicate.html","title":"Instream.Deleter.Predicate","type":"module"},{"doc":"Encoder for the InfluxDB line protocol.","ref":"Instream.Encoder.Line.html","title":"Instream.Encoder.Line","type":"module"},{"doc":"Creates protocol contents for a list of data points.","ref":"Instream.Encoder.Line.html#encode/1","title":"Instream.Encoder.Line.encode/1","type":"function"},{"doc":"","ref":"Instream.Encoder.Line.html#t:point/0","title":"Instream.Encoder.Line.point/0","type":"type"},{"doc":"Behaviour definition for switchable HTTP clients.","ref":"Instream.HTTPClient.html","title":"Instream.HTTPClient","type":"behaviour"},{"doc":"","ref":"Instream.HTTPClient.html#c:request/5","title":"Instream.HTTPClient.request/5","type":"callback"},{"doc":"","ref":"Instream.HTTPClient.html#t:method/0","title":"Instream.HTTPClient.method/0","type":"type"},{"doc":"","ref":"Instream.HTTPClient.html#t:response/0","title":"Instream.HTTPClient.response/0","type":"type"},{"doc":"HTTP client using :hackney.request/5 . See Instream.Connection.Config and Instream.HTTPClient if you want to replace the client with a library of your choice.","ref":"Instream.HTTPClient.Hackney.html","title":"Instream.HTTPClient.Hackney","type":"module"},{"doc":"Default logger for all entries.","ref":"Instream.Log.DefaultLogger.html","title":"Instream.Log.DefaultLogger","type":"module"},{"doc":"Logs a request.","ref":"Instream.Log.DefaultLogger.html#log/1","title":"Instream.Log.DefaultLogger.log/1","type":"function"},{"doc":"Log entry definition for delete requests.","ref":"Instream.Log.DeleteEntry.html","title":"Instream.Log.DeleteEntry","type":"module"},{"doc":"","ref":"Instream.Log.DeleteEntry.html#t:t/0","title":"Instream.Log.DeleteEntry.t/0","type":"type"},{"doc":"Metadata common to all log entries.","ref":"Instream.Log.Metadata.html","title":"Instream.Log.Metadata","type":"module"},{"doc":"","ref":"Instream.Log.Metadata.html#t:t/0","title":"Instream.Log.Metadata.t/0","type":"type"},{"doc":"Log entry definition for ping requests.","ref":"Instream.Log.PingEntry.html","title":"Instream.Log.PingEntry","type":"module"},{"doc":"","ref":"Instream.Log.PingEntry.html#t:t/0","title":"Instream.Log.PingEntry.t/0","type":"type"},{"doc":"Log entry definition for query requests.","ref":"Instream.Log.QueryEntry.html","title":"Instream.Log.QueryEntry","type":"module"},{"doc":"","ref":"Instream.Log.QueryEntry.html#t:t/0","title":"Instream.Log.QueryEntry.t/0","type":"type"},{"doc":"Log entry definition for status requests.","ref":"Instream.Log.StatusEntry.html","title":"Instream.Log.StatusEntry","type":"module"},{"doc":"","ref":"Instream.Log.StatusEntry.html#t:t/0","title":"Instream.Log.StatusEntry.t/0","type":"type"},{"doc":"Log entry definition for write requests.","ref":"Instream.Log.WriteEntry.html","title":"Instream.Log.WriteEntry","type":"module"},{"doc":"","ref":"Instream.Log.WriteEntry.html#t:t/0","title":"Instream.Log.WriteEntry.t/0","type":"type"},{"doc":"Series Definition Series definitions can be used to have a fixed structured usable for reading and writing data to an InfluxDB server: defmodule MySeries.CPULoad do use Instream.Series series do measurement &quot;cpu_load&quot; tag :host , default : &quot;www&quot; tag :core field :value , default : 100 field :value_desc end end The macros tag/2 and field/2 both accept a keyword tuple with a :default entry. This value will be pre-assigned when using the data struct with all other fields or tags being set to nil . Structs Each of your series definitions will register three separate structs. Based on the aforementioned MySeries.CPULoad you will have access to the following structs: % MySeries.CPULoad { fields : % MySeries.CPULoad.Fields { value : 100 , value_desc : nil } , tags : % MySeries.CPULoad.Tags { host : &quot;www&quot; , core : nil } , timestamp : nil } :timestamp is expected to be either a unix nanosecond or an RFC3339 timestamp. Compile-Time Series Validation Defining a series triggers a validation function during compilation. This validation for example prevents the usage of a field and tag sharing the same name. Some internal keys like :time will also raise an ArgumentError during compilation. You can deactivate this compile time validation by passing skip_validation: true in your series module: defmodule MySeries.ConflictButAccepted do use Instream.Series , skip_validation : true series do tag :conflict field :conflict end end Validations performed: having use Instream.Series requires also calling series do .. end a measurement must be defined at least one field must be defined fields and tags must not share a name the names :_field , :_measurement , and :time are not allowed to be used for fields or tags Reading Series Points (Hydration) Whenever you want to convert a plain map or a query result into a specific series you can use the built-in hydration methods: MySeries . from_map ( %{ timestamp : 1_234_567_890 , some_tag : &quot;hydrate&quot; , some_field : 123 } ) ~S(SELECT * FROM &quot;my_measurement&quot;) |&gt; MyConnection . query ( ) |&gt; MySeries . from_result ( ) The timestamp itself is kept &quot;as is&quot; for integer values, timestamps in RFC3339 format (e.g. &quot;1970-01-01T01:00:00.000+01:00&quot; ) will be converted to :nanosecond integer values. Please be aware that when using an OTP release prior to 21.0 the time will be truncated to :microsecond precision due to :calendar.rfc3339_to_system_time/2 not being available and DateTime.from_iso8601/1 only supporting microseconds. Hydrating Results with Multiple Fields (InfluxDB v2.x Pivoting) If you are connecting to an InfluxDB v2.x instance and have more than one field in your series you will need to &quot;pivot&quot; your query results to be able to fully hydrate your structs: MyConnection . query ( ~s( from(bucket: &quot; \#{ MyConnection . config ( :bucket ) } &quot;) |&gt; range ( start : - 5 m ) |&gt; pivot ( rowKey : [ &quot;_time&quot; ] , columnKey : [ &quot;_field&quot; ] , valueColumn : &quot;_value&quot; ) ) ) Without using the pivot() function in your :flux query you will receive one result struct for each field (other fields containing default values) instead of one struct with all fields. See https://www.influxdata.com/blog/how-to-pivot-data-flux-columnar-data/ for more details and examples. Writing Series Points You can then use your series module to assemble a data point (one at a time) for writing: data = % MySeries { } data = %{ data | fields : %{ data . fields | value : 17 } } data = %{ data | tags : %{ data . tags | bar : &quot;bar&quot; , foo : &quot;foo&quot; } } And then write one or many at once: MyConnection . write ( point ) MyConnection . write ( [ point_1 , point_2 , point_3 ] ) If you want to pass an explicit timestamp you can use the key :timestamp : data = % MySeries { } data = %{ data | timestamp : 1_439_587_926_000_000_000 } The timestamp is (by default) expected to be a nanosecond unix timestamp. To use different precision (for all points in this write operation!) you can change this value by modifying your write call: data = % MySeries { } data = %{ data | timestamp : 1_439_587_926 } MyConnection . write ( data , precision : :second ) Supported precision types are: :hour :minute :second :millisecond :microsecond :nanosecond :rfc3339 Please be aware that the UDP protocol writer ( Instream.Writer.UDP ) does not support custom timestamp precisions. All UDP timestamps are implicitly expected to already be at nanosecond precision.","ref":"Instream.Series.html","title":"Instream.Series","type":"behaviour"},{"doc":"Provides additional metadata for a series. Available Information :fields : the fields in the series :measurement : the measurement of the series :tags : the available tags defining the series","ref":"Instream.Series.html#c:__meta__/1","title":"Instream.Series.__meta__/1","type":"callback"},{"doc":"Defines a field in the series.","ref":"Instream.Series.html#field/2","title":"Instream.Series.field/2","type":"macro"},{"doc":"Creates a series dataset from any given map. Keys not defined in the series are silently dropped.","ref":"Instream.Series.html#c:from_map/1","title":"Instream.Series.from_map/1","type":"callback"},{"doc":"Creates a list of series datasets from a query result. Keys not defined in the series are silently dropped.","ref":"Instream.Series.html#c:from_result/1","title":"Instream.Series.from_result/1","type":"callback"},{"doc":"Defines the measurement of the series.","ref":"Instream.Series.html#measurement/1","title":"Instream.Series.measurement/1","type":"macro"},{"doc":"Defines the series.","ref":"Instream.Series.html#series/1","title":"Instream.Series.series/1","type":"macro"},{"doc":"Defines a tag in the series.","ref":"Instream.Series.html#tag/2","title":"Instream.Series.tag/2","type":"macro"},{"doc":"Point writer behaviour.","ref":"Instream.Writer.html","title":"Instream.Writer","type":"behaviour"},{"doc":"Writes a point.","ref":"Instream.Writer.html#c:write/3","title":"Instream.Writer.write/3","type":"callback"},{"doc":"Optional list of workers to be supervised by the connection.","ref":"Instream.Writer.html#c:writer_workers/1","title":"Instream.Writer.writer_workers/1","type":"callback"},{"doc":"Point writer for the line protocol. Write Options precision : write points with a precision other than :nanosecond InfluxDB v2.x Options bucket : write data to a specific bucket org : write data to a specific organization InfluxDB v1.x Options database : write data to a specific database retention_policy : write data with a specific retention policy","ref":"Instream.Writer.Line.html","title":"Instream.Writer.Line","type":"module"},{"doc":"Point writer for the line protocol using UDP. Configuration Write queries are run through a process pool having an additional timeout: config :my_app , MyConnection , pool : [ max_overflow : 10 , size : 5 ] , pool_timeout : 500 This configuration will be used to wait for an available worker to execute a query and defaults to 5_000 . Write Options async: true : execute writes asynchronously","ref":"Instream.Writer.UDP.html","title":"Instream.Writer.UDP","type":"module"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Enhancements Initial query/write support for InfluxDB 2.x added Logging configuration fetched at runtime instead of during compilation The HTTP client library can be replaced by providing a custom implementation of the Instream.HTTPClient behaviour Backwards incompatible changes Minimum required elixir version is now ~&gt; 1.9 Passing a host to ping/1,2 , status/1,2 , version/1,2 is now longer supported. Only the hostname configured in the connection module will be used. Support for the InfluxDB versions earlier than 1.7.x is no longer guaranteed","ref":"changelog.html#2-0-0-2022-06-18","title":"Changelog - 2.0.0 (2022-06-18)","type":"extras"},{"doc":"Enhancements Documentation is now available inline ( @moduledoc , ...) with the README.md file targeting the repository (development) instead of releases Hydrating query results into series structs now parses RFC3339 times into :nanosecond timestamps. If you are using OTP 21.0 or later you will get the full precision, earlier versions are truncated to the :microsecond precision supported by DateTime ! Read queries (everything except writes) are now executed directly in the calling process Sending asynchronous queries is now limited to write queries Writers can modify the worker state during init and terminate by implementing the callbacks init_worker/1 and terminate_worker/1 Backwards incompatible changes All &quot;administrative query modules&quot; have been removed Minimum required elixir version is now ~&gt; 1.7 Support for accessing the system environment for configuration has been removed in favor of initializer functions/modules Support for implementing use Instream.Writer has been removed in favor of @behaviour Instream.Writer Support for plural time units (e.g. :seconds instead of :second ) has been removed The default JSON decoder has been switched from :poison to :jason The query builder has been removed Bug fixes :influxql has been moved from :included_applications to :applications to avoid potential problems with release tooling ( #53 )","ref":"changelog.html#v1-0-0-2020-09-06","title":"Changelog - v1.0.0 (2020-09-06)","type":"extras"},{"doc":"Enhancements Errors from :hackney.body/1 are now passed to the caller instead of raising a MatchError Unix socket usage has been documented","ref":"changelog.html#v0-22-0-2019-11-29","title":"Changelog - v0.22.0 (2019-11-29)","type":"extras"},{"doc":"Enhancements Read queries can now use parameter binding The configuration option :json_encoder for the library used for JSON encoding has been added","ref":"changelog.html#v0-21-0-2019-05-15","title":"Changelog - v0.21.0 (2019-05-15)","type":"extras"},{"doc":"Enhancements Queries can be sent as Flux language queries to InfluxDB using [query_language: :flux] in the connection or query options Soft deprecations (no warnings) The query builder has been removed from documentation and will be eventually removed completely. This will be done because of the current limitations of the builder implementation and the InfluxDB move from InfluxQL to Flux as the query language of choice Deprecations Accessing the system environment by configuring {:system, var} or {:system, var, default} will now result in a Logger.info/1 message and will stop working in a future release Implementing use Instream.Writer has been deprecated in favor of the more explicit @behaviour Instream.Writer . Old implementations will trigger a compile time warning until the old macro is removed The already (soft) deprecated time units with plural names (e.g. :seconds ) will now issue Logger.info/1 messages when used Backwards incompatible changes Support for the InfluxDB versions earlier than 1.4.x is no longer guaranteed","ref":"changelog.html#v0-20-0-2019-04-19","title":"Changelog - v0.20.0 (2019-04-19)","type":"extras"},{"doc":"Enhancements Connections can be declared with compile time configuration defaults that are later overwritten by the application environment The library used for JSON decoding can be changed by using the :json_decoder configuration Soft deprecations (no warnings) All &quot;administrative query modules&quot; have been removed from the documentation. They will eventually be removed completely after a proper deprecation phase Support for {:system, &quot;ENV_VARIABLE&quot;} configuration has been removed from the documentation. It will eventually be removed completely after a proper deprecation phase Backwards incompatible changes Default configuration has been extended with host: &quot;localhost&quot; Minimum required elixir version is now ~&gt; 1.5 Missing application configuration will no longer raise during compilation Public access to the internal pool module name has been removed","ref":"changelog.html#v0-19-0-2019-01-24","title":"Changelog - v0.19.0 (2019-01-24)","type":"extras"},{"doc":"Enhancements Configuration can be done on connection (re-) start by setting a {mod, fun} tuple for the config key :init . This method will be called with the connection module name as the first (and only) parameter and is expected to return :ok Experimental support to convert plain maps or query results into series structs has been added Passwords are automatically redacted from logged queries when using the default logger Supporting rules for the elixir formatter are available Bug fixes Special characters in url parameter values (like &amp; or ? in a query) are now properly escaped ( #43 ) Backwards incompatible changes Quoting of identifiers/values has been extracted to the package influxql ( hex )","ref":"changelog.html#v0-18-0-2018-08-11","title":"Changelog - v0.18.0 (2018-08-11)","type":"extras"},{"doc":"Enhancements Support for Elixir 1.5 style child_spec has been added ( #39 )","ref":"changelog.html#v0-17-1-2017-12-16","title":"Changelog - v0.17.1 (2017-12-16)","type":"extras"},{"doc":"Backwards incompatible changes Minimum required elixir version is now ~&gt; 1.3","ref":"changelog.html#v0-17-0-2017-12-10","title":"Changelog - v0.17.0 (2017-12-10)","type":"extras"},{"doc":"Enhancements Queries can be executed with an additional option :pool_timeout used to wait for an available worker process. This option is independent of other configured/passed timeouts System environment configuration can set an optional default value to be used if the environment variable is unset Bug fixes All queries now use a GenServer.call/3 timeout of :infinity . This allows raising the timeouts for a call above the default of 5_000 ( #38 )","ref":"changelog.html#v0-16-0-2017-09-25","title":"Changelog - v0.16.0 (2017-09-25)","type":"extras"},{"doc":"Enhancements Every connection method (like read or write ) can set per-call http_opts passed on to the hackney client Querying for data can now return an {:error, term} style tuple if the communication with the server failed. For example {:error, :nxdomain} if the host that is queried cannot be resolved by hackney ( #33 ) The line writer now accepts a retention policy used for writing ( #34 ) The line writer will now return an {:error, term} style tuple if a problem occurs. For example {:error, :nxdomain} if the host that should receive data cannot be resolved by hackney ( #33 ) Timeouts occurring when executing a query are now returned as {:error, :timeout} instead of raising ( #33 ) Backwards incompatible changes Series definitions raise upon compilation if the contain a tag and a field with the same name. This is done to prevent the InfluxDB behavior of adding _1 to such fields when storing them","ref":"changelog.html#v0-15-0-2017-03-16","title":"Changelog - v0.15.0 (2017-03-16)","type":"extras"},{"doc":"Enhancements Connections can be configured with a default database used when no value is found in method call or series definition Fields and tags with the name :time will raise after compilation. They are unqueryable and will be dropped by InfluxDB v1.1.0 when trying to write them. Queries can return CSV formatted responses from the server when running InfluxDB &gt;= 1.1.0 Series definitions are now automatically associated with typespecs Backwards incompatible changes Minimum required elixir version is now ~&gt; 1.2 Minimum required erlang version is now ~&gt; 18.0 Query builder support for IF NOT EXISTS has been removed Series definitions now require to contain at least one field Soft deprecations (no warnings) The plural names for precision units have been soft deprecated to be in line with the similar new naming in Erlang 19.1 and the upcoming Elixir 1.4.0 . The old types will continue to work but their usage is discouraged. They will be completely deprecated in an upcoming release.","ref":"changelog.html#v0-14-0-2016-12-18","title":"Changelog - v0.14.0 (2016-12-18)","type":"extras"},{"doc":"Enhancements Configuration has been split into &quot;compile time&quot; and &quot;runtime&quot; parts. This readds the possibility to change configuration values without recompiling the connection modules ( #22 ) Configuration values can be fetched from the system environment using {:system, ENV_VAR} Timeouts for queries (individual and connection wide) can be configured ( #21 ) Query builder supports LIMIT and OFFSET for queries ( #19 ) Writing a series struct with one or more empty tags will now properly construct an entry without these tags present Soft deprecations (no warnings) The precision units :micro_seconds , :milli_seconds and :nano_seconds have been renamed to :microseconds , :milliseconds and :nanoseconds to matches the upcoming System.time_unit definition of Elixir 1.3.0 . Old variables will continue to work but are highly discouraged and will be completely deprecated in an upcoming release. Deprecations Defining a series without fields is deprecated and will raise in a future version Backwards incompatible changes Configuring a connection with multiple hosts is no longer supported The otp app of a connection is no longer returned when calling MyConnection.config/0,1 . It is now only available by specifically requesting it via MyConnection.config([:otp_app]) Writer modules now get a map passed with the connection module registered at key :module and (if configured) a udp socket for writing at :udp_socket The Cluster namespace of queries has been removed","ref":"changelog.html#v0-13-0-2016-09-11","title":"Changelog - v0.13.0 (2016-09-11)","type":"extras"},{"doc":"Enhancements Namespace of administrational convenience modules has been changed from Cluster to the better matching Admin Deprecations Configuring a connection with multiple hosts has been deprecated. Instead of multiple :hosts only a single :host is now expected The Cluster namespace of queries has been changed to Admin . For some backwards compatibility the old modules are delegated to the new ones until removed in the next release Backwards incompatible changes Atoms for defining a series' database are no longer supported Atoms for defining a series' measurement are no longer supported Support for the JSON protocol has been removed matching its removal in InfluxDB v0.12.0 Support for the SHOW SERVERS statement has been removed matching its removal in InfluxDB v0.13.0","ref":"changelog.html#v0-12-0-2016-05-13","title":"Changelog - v0.12.0 (2016-05-13)","type":"extras"},{"doc":"Enhancements Hackney options can be configured ( #17 ) Line writer now prefers a database passed via arguments over the one passed in the datapoint payload Logging for requests is available Pings can be send to specific servers Status requests can be send to a cluster Bug fixes Series are now compiled with full environment information preventing &quot;/path/to/lib/nofile&quot; to be compiled as the source of MySeries.Fields or MySeries.Tags Backwards incompatible changes Runtime connection configuration reading has been removed in favor of compile time inlining","ref":"changelog.html#v0-11-0-2016-04-14","title":"Changelog - v0.11.0 (2016-04-14)","type":"extras"},{"doc":"Enhancements Default precision ( :rfc3339 ) can be explicitly set Query builder can construct CREATE DATABASE statements Query builder can construct CREATE RETENTION POLICY statements Query builder can construct DROP DATABASE statements Query builder can construct DROP RETENTION POLICY statements Query builder can construct SHOW statements Series database definitions allow anything &quot;evaluating to a string&quot; ( #14 ) Series measurement definitions allow anything &quot;evaluating to a string&quot; ( #14 ) Deprecations Atoms for defining a series' database are deprecated Atoms for defining a series' measurement are deprecated Using if_not_exists ( CREATE DATABASE ) has been deprecated and will be completely removed once InfluxDB removes it Backwards incompatible changes :cluster query type has been removed in favor of regular :read queries Retention policy convenience function has been replaced with a more detailed version","ref":"changelog.html#v0-10-0-2016-02-27","title":"Changelog - v0.10.0 (2016-02-27)","type":"extras"},{"doc":"Enhancements Error messages are returned without leading/trailing whitespace Experimental query builder Incomplete or missing series definitions raise an error during compilation Ping a connection using Instream.Connection.ping/0 Reading queries can be executed directly using Instream.Connection.query/2 Writing query can be executed directly using Insream.Connection.write/2 Deprecations Using Instream.Data.Read or Instream.Data.Write directly is discouraged and will eventually be removed Backwards incompatible changes The measurement of a series is no longer optional The parameter opts for Instream.Data.Read and Instream.Data.Write is no longer optional use -ing Instream.Series without a complete series definition raises during compilation","ref":"changelog.html#v0-9-0-2015-12-27","title":"Changelog - v0.9.0 (2015-12-27)","type":"extras"},{"doc":"Enhancements Batch writing using Line and UDP writers ( #10 ) Fields can be defined with default values Tags can be defined with default values Writing can be done over UDP using Instream.Writer.UDP","ref":"changelog.html#v0-8-0-2015-11-18","title":"Changelog - v0.8.0 (2015-11-18)","type":"extras"},{"doc":"Enhancements Convenience module for &quot;SHOW DIAGNOSTICS&quot; queries Convenience module for &quot;SHOW STATS&quot; queries Precision (= epoch) can be passed to read queries Precision can be passed to write queries Bug fixes Pool configuration (size, overflow) is now properly taken from configuration Deprecations Support of the JSON protocol is deprecated and will be removed once InfluxDB removes it","ref":"changelog.html#v0-7-0-2015-10-22","title":"Changelog - v0.7.0 (2015-10-22)","type":"extras"},{"doc":"Enhancements IF NOT EXISTS can be passed to database creation queries Points can be written with explicit timestamps ( #8 ) Switched default write method to the line protocol Tags are optional when writing with the line protocol Backwards incompatible changes Switched default write method to the line protocol","ref":"changelog.html#v0-6-0-2015-09-27","title":"Changelog - v0.6.0 (2015-09-27)","type":"extras"},{"doc":"Enhancements Convenience module for SHOW SERVERS queries Queries can be executed asynchronously Support for line protocol Backwards incompatible changes Write queries return :ok instead of nil","ref":"changelog.html#v0-5-0-2015-09-05","title":"Changelog - v0.5.0 (2015-09-05)","type":"extras"},{"doc":"Enhancements Allows using header authentication (basic auth) Allows using pre-defined series modules for write queries Provides a way to define series as a module (struct) Backwards incompatible changes Authentication uses headers by default","ref":"changelog.html#v0-4-0-2015-07-25","title":"Changelog - v0.4.0 (2015-07-25)","type":"extras"},{"doc":"Enhancements Allows managing retention policies Dependencies not used in production builds are marked as optional Bug fixes Authentication is passed using query parameters by default Backwards incompatible changes Auth configuration is now expected to be a Keyword.t Queries of type :host are now of type :cluster Remapped Admin namespace to Cluster to match query types","ref":"changelog.html#v0-3-0-2015-06-19","title":"Changelog - v0.3.0 (2015-06-19)","type":"extras"},{"doc":"Enhancements Allows accessing raw query results (undecoded binaries) using Instream.Connection.execute/2 Read queries (binaries) can be executed on a database Write queries (map data) can be executed on a database","ref":"changelog.html#v0-2-0-2015-04-19","title":"Changelog - v0.2.0 (2015-04-19)","type":"extras"},{"doc":"Initial Release","ref":"changelog.html#v0-1-0-2015-02-23","title":"Changelog - v0.1.0 (2015-02-23)","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html","title":"License","type":"extras"},{"doc":"InfluxDB driver for Elixir","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Tested InfluxDB versions: 1.7.11 1.8.10 2.0.9 2.1.1 2.2.0 2.3.0","ref":"readme.html#influxdb-support","title":"Overview - InfluxDB Support","type":"extras"},{"doc":"To use Instream with your projects, edit your mix.exs file and add the required dependencies: defp deps do [ # ... { :instream , &quot;~&gt; 2.0&quot; } , # ... ] end Testing To run the tests you need to have HTTP authentication enabled. The following environment variables are used to select some test suites and the InfluxDB version under test: INFLUXDB_VERSION : one of &quot;1.7&quot; , &quot;1.8&quot; , &quot;2.0&quot; , &quot;2.1&quot; , or &quot;2.2&quot; INFLUXDB_PORT_UDP : the UDP port used for writer testing (InfluxDB 1.x only) INFLUXDB_SOCKET : path to the InfluxDB unix socket (InfluxDB 1.8 only) INFLUXDB_TOKEN : the authentication token used (InfluxDB 2.x only)","ref":"readme.html#package-setup","title":"Overview - Package Setup","type":"extras"},{"doc":"Connections To connect to an InfluxDB server you need a connection module: defmodule MyConnection do use Instream.Connection , otp_app : :my_app end The :otp_app name and the name of the module can be freely chosen but have to be linked to a corresponding configuration entry. This defined connection module needs to be hooked up into your supervision tree: children = [ # ... MyConnection , # ... ] Example of the matching configuration entry: # InfluxDB v2.x config :my_app , MyConnection , auth : [ method : :token , token : &quot;my_token&quot; ] , bucket : &quot;my_default_bucket&quot; , org : &quot;my_default_org&quot; , host : &quot;my.influxdb.host&quot; , version : :v2 # InfluxDB v1.x config :my_app , MyConnection , auth : [ username : &quot;my_username&quot; , password : &quot;my_password&quot; ] , database : &quot;my_default_database&quot; , host : &quot;my.influxdb.host&quot; More details on connections and configuration options can be found with the Instream.Connection and Instream.Connection.Config modules. Queries # Flux query MyConnection . query ( &quot;&quot;&quot; from(bucket: &quot; \#{ MyConnection . config ( :bucket ) } &quot;) |&gt; range(start: -5m) |&gt; filter(fn: (r) =&gt; r._measurement == &quot;instream_examples&quot; ) |&gt; first() &quot;&quot;&quot; ) # InfluxQL query MyConnection . query ( &quot;SELECT * FROM instream_examples&quot; ) A more detailed documentation on queries (reading/writing/options) is available in the documentation for the modules Instream and Instream.Connection .","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"If you do not want to define the raw maps for writing data you can pre-define a series for later usage: defmodule MySeries do use Instream.Series series do measurement &quot;my_measurement&quot; tag :bar tag :foo field :value end end More information about series definitions can be found in the module documentation of Instream.Series . Writing Series Points You can then use this module to assemble a data point (one at a time) for writing: data = % MySeries { } data = %{ data | fields : %{ data . fields | value : 17 } } data = %{ data | tags : %{ data . tags | bar : &quot;bar&quot; , foo : &quot;foo&quot; } } And then write one or many at once: MyConnection . write ( point ) MyConnection . write ( [ point_1 , point_2 , point_3 ] )","ref":"readme.html#series-definitions","title":"Overview - Series Definitions","type":"extras"},{"doc":"Apache License, Version 2.0","ref":"readme.html#license","title":"Overview - License","type":"extras"}]